CREATE TABLE universes (
	id TEXT PRIMARY KEY NOT NULL,
	name TEXT NOT NULL UNIQUE
);

CREATE TABLE projects (
	id TEXT PRIMARY KEY NOT NULL,
	universe_id TEXT NOT NULL,
	name TEXT NOT NULL,

	FOREIGN KEY (universe_id)
		REFERENCES universes (id)
		ON DELETE CASCADE
);

CREATE TABLE targets (
	id TEXT PRIMARY KEY NOT NULL,
	project_id TEXT NOT NULL,
	name TEXT NOT NULL,
	target_type TEXT NOT NULL,

	FOREIGN KEY (project_id)
		REFERENCES projects (id)
		ON DELETE CASCADE,

	CHECK (target_type in (
		'test',
		'workflow1'
	)),

	UNIQUE (project_id, name)
);

CREATE Table workflow1_target_configurations (
	id TEXT PRIMARY KEY NOT NULL,
	target_id TEXT NOT NULL UNIQUE,
	workflow_name TEXT NOT NULL,

	FOREIGN KEY (target_id)
		REFERENCES targets (id)
		ON DELETE CASCADE
);

CREATE TABLE alerts (
	id TEXT PRIMARY KEY NOT NULL,
	project_id TEXT NOT NULL,
	name TEXT NOT NULL,
	enabled BOOLEAN NOT NULL,
	query TEXT NOT NULL,
	query_period DOUBLE PRECISION NOT NULL,
	min_notification_interval DOUBLE PRECISION NOT NULL,
	mute_until DOUBLE PRECISION NOT NULL,
	last_evaluated DOUBLE PRECISION NOT NULL,
	last_triggered_time DOUBLE PRECISION NOT NULL,
	consecutive_failures INTEGER NOT NULL DEFAULT 0,

	UNIQUE (project_id, name),

	FOREIGN KEY (project_id)
		REFERENCES projects (id)
		ON DELETE CASCADE
);

create table alert_targets (
	alert_id TEXT NOT NULL,
	target_id TEXT NOT NULL,

	PRIMARY KEY (alert_id, target_id),

	FOREIGN KEY (alert_id)
		REFERENCES alerts (id)
		ON DELETE CASCADE,

	FOREIGN KEY (target_id)
		REFERENCES targets (id)
		ON DELETE CASCADE
);

create table alert_triggers (
	id TEXT PRIMARY KEY NOT NULL,
	alert_id TEXT NOT NULL,
	comparison_type TEXT NOT NULL,
	warning_threshold DOUBLE PRECISION NOT NULL,
	critical_threshold DOUBLE PRECISION NOT NULL,
	aggregation_column TEXT NOT NULL,
	aggregation_index INTEGER NOT NULL,

	FOREIGN KEY (alert_id)
		REFERENCES alerts (id)
		ON DELETE CASCADE,

	CHECK (comparison_type IN (
		'ge',
		'le'
	))
);

CREATE TABLE alert_evaluations (
	id INTEGER PRIMARY KEY NOT NULL,
	uuid TEXT NOT NULL UNIQUE,
	alert_id TEXT NOT NULL,
	evaluated_at DOUBLE PRECISION NOT NULL,
	evaluation_start DOUBLE PRECISION NOT NULL,
	evaluation_end DOUBLE PRECISION NOT NULL,
	query TEXT,
	example_groups TEXT NOT NULL,
	error TEXT,
	warning_trigger_count INTEGER NOT NULL DEFAULT 0,
	critical_trigger_count INTEGER NOT NULL DEFAULT 0,
	warning_group_count INTEGER NOT NULL DEFAULT 0,
	critical_group_count INTEGER NOT NULL DEFAULT 0,

	FOREIGN KEY (alert_id)
		REFERENCES alerts (id)
		ON DELETE SET NULL
);

create table notifications (
	id INTEGER PRIMARY KEY NOT NULL,
	uuid TEXT NOT NULL UNIQUE,
	alert_evaluation_id TEXT NOT NULL,
	created_at DOUBLE PRECISION NOT NULL,
	materialized_alert TEXT NOT NULL,
	state TEXT NOT NULL,

	FOREIGN KEY (alert_evaluation_id)
		REFERENCES alert_evaluations (uuid),

	CHECK (state in (
		'pending',
		'complete'
	))
);

CREATE INDEX notifications_created_at ON notifications(created_at);

CREATE TABLE pending_notification_targets (
	notification_id BIGINT NOT NULL,
	target_id TEXT NOT NULL,
	permanent_error_count INTEGER NOT NULL,
	transient_error_count INTEGER NOT NULL,
	last_tried_at DOUBLE PRECISION NOT NULL,
	last_error_message TEXT,

	PRIMARY KEY (target_id, notification_id),

	FOREIGN KEY (notification_id)
		REFERENCES notifications (id)
		ON DELETE CASCADE,

	FOREIGN KEY (target_id)
		REFERENCES targets (id)
		ON DELETE CASCADE
);

CREATE TABLE completed_notification_targets (
	notification_id BIGINT NOT NULL,
	target_id TEXT NOT NULL,
	completed_at DOUBLE PRECISION NOT NULL,
	error_message TEXT,

	PRIMARY KEY (target_id, notification_id),

	FOREIGN KEY (notification_id)
		REFERENCES notifications (id)
		ON DELETE CASCADE,

	FOREIGN KEY (target_id)
		REFERENCES targets (id)
		ON DELETE CASCADE
);
